local _G = getfenv(0)
local GameTooltip = _G.GameTooltip
local unitTags, layout, layoutName, lastUpdate = {}, {}

local RantTooltip = {}
RantTooltip.name = ...
RantTooltip.version = 0.28

RantTooltip.cache = { talents = {}, spec = {} }

RantTooltip["Anchor"] = { "CURSOR", 0, 0 }

RantTooltip["ClassColors"] = {}
for class, color in pairs(RAID_CLASS_COLORS) do
	RantTooltip.ClassColors[class] = {color.r, color.g, color.b}
end

RantTooltip["ReactionColors"] = {}
for reaction, color in pairs(FACTION_BAR_COLORS) do
	RantTooltip.ReactionColors[reaction] = {color.r, color.g, color.b}
end

RantTooltip["PowerColors"] = {}
for power, color in pairs(PowerBarColor) do
	RantTooltip.PowerColors[power] = {color.r, color.g, color.b}
end

RantTooltip["OnLoad"] = function(self)
	GameTooltipStatusBar:SetAlpha(0)
	GameTooltipStatusBar:SetStatusBarTexture(nil)

	GameTooltip_SetDefaultAnchor = function(self, parent)
		if GameTooltip.OverrideSetDefaultAnchor then
			GameTooltip.OverrideSetDefaultAnchor(self, parent)
		else
			if RantTooltip.Anchor[1] == "CURSOR" or RantTooltip.Anchor[1] == "PRESERVE" then
				self:SetOwner(parent,"ANCHOR_"..RantTooltip.Anchor[1])
			else
				self:SetOwner(parent,"ANCHOR_NONE")
				self:SetPoint(unpack(RantTooltip.Anchor))
			end
		end
	end
	
	RantTooltip:RegisterScript("GameTooltip", "OnShow")
	RantTooltip:RegisterScript("GameTooltip", "OnHide")
	RantTooltip:RegisterScript("GameTooltip", "OnTooltipSetUnit")
	RantTooltip:RegisterScript("GameTooltip", "OnUpdate")
	RantTooltip:RegisterScript("GameTooltip", "OnEvent")
	
	RantTooltip:RegisterScript("GameTooltipStatusBar", "OnShow", RantTooltip.UpdateHealthBar)
	RantTooltip:RegisterScript("GameTooltipStatusBar", "OnValueChanged", RantTooltip.UpdateHealthBar)

	GameTooltip:RegisterEvent("PLAYER_LOGIN")
end

RantTooltip["RegisterScript"] = function(self, obj, handler, func)
	local realObj = type(obj) == "string" and _G[obj] or obj
	if realObj then
		func = func or type(obj) == "string" and RantTooltip[obj..":"..handler]
		if realObj:GetScript(handler) then
			realObj:HookScript(handler, func)
		else
			realObj:SetScript(handler, func)
		end
	end
end

RantTooltip["RegisterLayout"] = function(self, name, func)
	if not layout[name] then
		layout[name] = func
	end
end

RantTooltip["UseLayout"] = function(self, name)
	layoutName = name
	RantTooltip:Initialize()
end

RantTooltip["SkinTooltip"] = function(self, tooltip)
	if layout[layoutName] then
		layout[layoutName](tooltip)
	end
	if tooltip:IsObjectType("GameTooltip") then
		self:RegisterScript(tooltip, "OnSizeChanged", function(self)
			if self.PreOnSizeChanged then
				self:PreOnSizeChanged()
			end
			if self.itemIcon then
				if self:GetItem() then
					local item = select(2,self:GetItem())
					self.itemIcon:SetTexture(GetItemIcon(item))
				else
					self.itemIcon:SetTexture(nil)
				end
			end
			if self.PostOnSizeChanged then
				self:PostOnSizeChanged()
			end
		end)
	end
end

RantTooltip["SetInitialAnchor"] = function(self, ...)
	if type(...) == "table" then
		self.initialAnchor = { unpack(...) }
		self.Anchor = { unpack(...) }
	else
		self.initialAnchor = { ... }
		self.Anchor = { ... }
	end
end

RantTooltip["Initialize"] = function(self)
	self:SkinTooltip(GameTooltip)
	self:SkinTooltip(ItemRefTooltip)
	self:SkinTooltip(ShoppingTooltip1)
	self:SkinTooltip(ShoppingTooltip2)
	self:SkinTooltip(ShoppingTooltip3)
	self:SkinTooltip(WorldMapTooltip)
	--self:SkinTooltip(DropDownList1MenuBackdrop)
	--self:SkinTooltip(DropDownList2MenuBackdrop)
	for _, bar in ipairs{GameTooltip.HealthBar, GameTooltip.PowerBar} do
		if bar and bar.Anchor then
			self:RegisterScript(bar, "OnShow", function()
				if self.Anchor[1] == "CURSOR" and bar.Anchor then
					self.Anchor[2] = bar.Anchor[1]
					self.Anchor[3] = bar.Anchor[2]
				end
			end)
		end
	end
end

RantTooltip["RGBHex"] = function(_, r, g, b)
	if type(r) == "table" then
		if r.r then
			r, g, b = r.r, r.g, r.b
		else
			r, g, b = unpack(r)
		end
	end
	return string.format("|cff%02x%02x%02x", (r or 1) * 255, (g or 1) * 255, (b or 1) * 255)
end

RantTooltip["GetUnitFaction"] = function(self, unit)
	if not UnitIsPlayer(unit) then
		for i = 1, GetNumFactions() do
			local faction = GameTooltip:FindLine(GetFactionInfo(i), true, 2)
			if faction then
				return faction:GetText()
			end
		end
	end
end

RantTooltip["GetUnitInfo"] = function(self, unit)
	local info
	for i = 2, GameTooltip:NumLines() do
		local line = _G["GameTooltipTextLeft"..i]
		local text = line:GetText()
		local guild = GetGuildInfo(unit)
		if (text and not text:match("^"..PVP.."$") and not text:match("^"..LEVEL.."%s%d-")
		and (not guild or not text:match("^"..guild.."$")) 
		and (not self.factionInfo or text ~= self.factionInfo)) then
			if not info then info = {} end
			table.insert(info, { text, line:GetTextColor() })
		end
	end
	return info
end

local shortNumber = function(num)
	if num > 1000000 then
		num = string.format("%.2f", (num / 1000000)).."m"
	elseif num > 100000 then
		num = string.format("%.0f", (num / 1000)).."k"
	end
	return num
end

RantTooltip["Tags"] = {
	["$name"] = function(unit) return RantTooltip:RGBHex(GameTooltip_UnitColor(unit))..UnitName(unit).."|r" end,
	["$tname"] = function(unit) return RantTooltip:RGBHex(GameTooltip_UnitColor(unit))..(UnitPVPName(unit) or UnitName(unit)).."|r" end,
	["$dnd"] = function(unit) return UnitIsAFK(unit) and "|cff00ff00<AFK>|r" or UnitIsDND(unit) and "|cff00ff00<DND>|r" end,
	["$server"] = function(unit) return (not UnitIsSameServer(unit, "player")) and RantTooltip:RGBHex(GameTooltip_UnitColor(unit))..select(2, UnitName(unit)).."|r" end,
	["$aserver"] = function(unit) return (not UnitIsSameServer(unit, "player")) and RantTooltip:RGBHex(GameTooltip_UnitColor(unit)).."*|r" end,
	["$guild"] = function(unit) return UnitIsPlayer(unit) and GetGuildInfo(unit) and "|cffff00ff"..GetGuildInfo(unit).."|r" or (RantTooltip.factionInfo and "|cffff00ff"..RantTooltip.factionInfo.."|r") end,
	["$rank"] = function(unit) return UnitIsPlayer(unit) and GetGuildInfo(unit) and "|cff00ffff"..select(2,GetGuildInfo(unit)).."|r" end,
	["$pvp"] = function(unit) 	return UnitIsPVP(unit) and "|cffff0000PvP|r" end,
	["$faction"] = function(unit) return RantTooltip.factionInfo and "|cffff00ff"..RantTooltip.factionInfo.."|r" end,
	["$target"] = function(unit)
		if UnitExists(unit.."target") then
			local color = RantTooltip:RGBHex(GameTooltip_UnitColor(unit.."target"))
			return (UnitIsUnit(unit.."target", unit) and "|cffffffff[SELF]|r" 
			or UnitIsUnit(unit.."target", "player") and "|cffffffff[YOU]|r" or color..UnitName(unit.."target").."|r")
		end
	end,
	["$level"] = function(unit) 
		local level = UnitLevel(unit)
		return level > 0 and RantTooltip:RGBHex(GetQuestDifficultyColor(level))..level.."|r" or "|cffff0000(??)|r"
	end,
	["$classification"] = function(unit)
		local class = UnitClassification(unit)
		class = (class == "worldboss" and " "..BOSS or class == "rareelite" and "!+" 
		or class == "elite" and "+" or class == "rare" and "!" or nil)
		if class then
			return RantTooltip:RGBHex(GetQuestDifficultyColor(UnitLevel(unit)))..class.."|r"
		end
	end,
	["$class"] = function(unit) return UnitIsPlayer(unit) and RantTooltip:RGBHex(GameTooltip_UnitColor(unit))..UnitClass(unit).."|r" end,
	["$race"] = function(unit) return UnitIsPlayer(unit) and "|cffffffff"..UnitRace(unit).."|r" end,
	["$type"] = function(unit) return (not UnitIsPlayer(unit)) and "|cffffffff"..(UnitCreatureType(unit) or "").."|r" end,
	["$talents"] = function(unit)
		local GUID = UnitGUID(unit)
		if RantTooltip.cache.talents[GUID] then
			return RantTooltip.cache.talents[GUID]
		elseif (UnitIsPlayer(unit) and CheckInteractDistance(unit, 1) 
		and (not UnitIsUnit("player", unit)) and UnitLevel(unit) > 9
		and (not UnitCanAttack("player", unit)) and (not UnitIsDead("player"))
		and ((not InspectFrame) or (InspectFrame and (not InspectFrame:IsShown())))) then
			local talents = {}
			if ((RantTooltip.TagEvents["$spec"].pos and not RantTooltip.TagEvents["$spec"].last
			or not RantTooltip.TagEvents["$spec"].pos) and not RantTooltip.TagEvents["$talents"].last) then
				NotifyInspect(unit)
				return "|cffffffffLoading|r"
			elseif (RantTooltip.TagEvents["$spec"].pos and RantTooltip.TagEvents["$spec"].last 
			and not RantTooltip.TagEvents["$talents"].last) then
				return "|cfffffffeLoading|r"
			end
			for i = 1, 3 do
				local num = select(3, GetTalentTabInfo(i, true))
				table.insert(talents, num)
			end
			if talents[1]+talents[2]+talents[3] == 0 then
				return "|cffffffffNone|r"
			end
			RantTooltip.cache.talents = table.wipe(RantTooltip.cache.talents)
			RantTooltip.cache.talents[GUID] = "|cffffffff"..string.join("/", unpack(talents)).."|r"
			return RantTooltip.cache.talents[GUID]
		end
	end,
	["$spec"] = function(unit)
		local GUID = UnitGUID(unit)
		if RantTooltip.cache.spec[GUID] then
			return RantTooltip.cache.spec[GUID]
		elseif RantTooltip.Tags["$talents"](unit) then
			local talents = {}
			if ((RantTooltip.TagEvents["$talents"].pos and not RantTooltip.TagEvents["$talents"].last
			or not RantTooltip.TagEvents["$talents"].pos) and not RantTooltip.TagEvents["$spec"].last) then
				NotifyInspect(unit)
				return "|cffffffffLoading|r"
			elseif (RantTooltip.TagEvents["$talents"].pos and RantTooltip.TagEvents["$talents"].last 
			and not RantTooltip.TagEvents["$spec"].last) then
				return "|cfffffffeLoading|r"
			end
			for i = 1, 3 do
				local num = select(3, GetTalentTabInfo(i, true))
				table.insert(talents, num)
			end
			if talents[1]+talents[2]+talents[3] == 0 then
				return "|cffffffffNone|r"
			end
			for num, tree in ipairs(talents) do
				if tree >= talents[1] and tree >= talents[2] and tree >= talents[3] then
					RantTooltip.cache.spec = table.wipe(RantTooltip.cache.spec)
					RantTooltip.cache.spec[GUID] = "|cffffffff"..GetTalentTabInfo(num, true).."|r"
					return RantTooltip.cache.spec[GUID]
				end
			end
		end
	end,
	["$misc"] = function(unit)
		if RantTooltip.unitInfo then
			local str
			for _, k in ipairs(RantTooltip.unitInfo) do
				if k[1] then
					str = (str and str.."\n" or "")..RantTooltip:RGBHex(k[2], k[3], k[4])..k[1].."|r"
				end
			end
			return str
		end
	end,
	["$cur"] = function(unit, bar)
		if bar then
			if bar == "Health" then
				local cur = GameTooltipStatusBar:GetValue()
				return cur <= 1 and floor(cur*100) or cur
			else
				return _G["Unit"..bar](unit)
			end
		end
	end,
	["$max"] = function(unit, bar)
		if bar then
			if bar == "Health" then
				local max = select(2,GameTooltipStatusBar:GetMinMaxValues())
				return max <= 1 and floor(max*100) or max
			else
				return _G["Unit"..bar.."Max"](unit)
			end
		end
	end,
	["$curshort"] = function(unit, bar)
		return shortNumber(RantTooltip.Tags["$cur"](unit, bar))
	end,
	["$maxshort"] = function(unit, bar)
		return shortNumber(RantTooltip.Tags["$max"](unit, bar))
	end,
	["$perc"] = function(unit, bar) 
		if bar then
			local hpcur, hpmax
			if bar == "Health" then
				hpcur = GameTooltipStatusBar:GetValue()
				hpmax = select(2,GameTooltipStatusBar:GetMinMaxValues())
			else
				hpcur = _G["Unit"..bar](unit)
				hpmax = _G["Unit"..bar.."Max"](unit)
			end
			return floor((hpcur/hpmax)*100)
		end
	 end,
	["$miss"] = function(unit, bar)
		if bar then
			local miss
			if bar == "Health" then
				miss = GameTooltipStatusBar:GetValue()-select(2,GameTooltipStatusBar:GetMinMaxValues())
				miss = miss ~= 0 and miss > -1 and floor(miss*100) or miss
			else
				miss = _G["Unit"..bar](unit)-_G["Unit"..bar.."Max"](unit)
			end
			return miss ~= 0 and "|cffff8080"..miss.."|r"
		end
	end,
}

RantTooltip["TagEvents"] = {
	["$target"] = { update = true },
	["$misc"] = { update = false },
	["$talents"] = { event = "INSPECT_TALENT_READY" },
	["$spec"] = { event = "INSPECT_TALENT_READY" },
}

RantTooltip["SetUnitTags"] = function(self, tags)
	for i, str in ipairs(tags) do
		for var in str:gmatch("$%a*") do
			if self.TagEvents[var] then
				self.TagEvents[var].str = str
				self.TagEvents[var].pos = i
				if self.TagEvents[var].event then
					GameTooltip:RegisterEvent(self.TagEvents[var].event)
				end
			end
		end
		for substr in str:gmatch("%b{}") do
			for var in substr:gmatch("$%a*") do
				if self.TagEvents[var] then
					self.TagEvents[var].substr = substr
				end
			end
		end
	end
	unitTags = tags
end

RantTooltip["GetUnitTags"] = function()
	return unitTags
end

RantTooltip["UpdateTag"] = function(self, var, tag)
	if not tag.pos then return end
	local unit = GameTooltip.unit
	local currentVar = self.Tags[var](unit)
	if tag.last ~= currentVar then
		if tag.last then
			local text = tag.substr or tag.str
			local lastVar = tag.last:gsub("[%[%(%]%)]", "%%%1")
			text = text:gsub(var, tag.last)
			text = self:ReplaceTags(text, unit):gsub("[%[%(%]%)]", "%%%1")
			local line = GameTooltip:FindLine(text)
			if currentVar then
				if line then
					line:SetText(line:GetText():gsub(lastVar, currentVar))
				end
			else
				if line then
					local lineText = line:GetText():gsub((tag.substr and text or lastVar), ""):trim()
					if lineText == "" then
						GameTooltip:DeleteLine(line)
					else
						line:SetText(lineText)
					end
				end
			end
		else
			local line
			for t in tag.str:gmatch("$%a*") do
				if t ~= var then
					local v = self.Tags[t](unit)
					if v and v ~= "" then
						line = GameTooltip:FindLine(v)
						if line then break end
					end
				end
			end
			local newLine = self:ReplaceTags(tag.str, unit)
			if line then
				line:SetText(newLine)
			else
				local found
				for k = tag.pos-1, 1, -1 do
					local above = self:ReplaceTags(unitTags[k], unit)
					local num = above and select(2,GameTooltip:FindLine(above, true))
					if num and GameTooltip:GetLine(num+1) then
						found = true
						GameTooltip:InsertLine(num+1, newLine)
						break
					end
				end
				if not found then
					GameTooltip:AddLine(newLine)
				end
			end
		end
		GameTooltip:Show()
		tag.last = currentVar
	end
end

RantTooltip["ReplaceTags"] = function(self, str, ...)
	for substr in str:gmatch("%b{}") do
		local conditionals, totalConditionals = 0, 0
		for var in substr:gmatch("$%a*") do
			totalConditionals = totalConditionals + 1
			if self.Tags[var] and self.Tags[var](...) then
				conditionals = conditionals + 1
			end
		end
		if conditionals == totalConditionals then
			str = str:gsub("[{}]", "", 2)
		else
			substr = substr:gsub("[%[%(%]%)]", "%%%1")
			str = str:gsub(substr, "")
		end
	end
	for var in str:gmatch("$%a*") do
		local val = self.Tags[var](...)
		str = str:gsub(var, val or "")
		if self.TagEvents[var] then
			self.TagEvents[var].last = val
		end
	end
	str = str:gsub("([%[%(<])(|c%x%x%x%x%x%x%x%x)(.-)(|r)([%]%)>])", "%2%1%4%2%3%4%2%5%4"):trim()
		--	:gsub("^(:)(%s)", "%1|r%2")
		--	:gsub("(|r)(:)(%s)", "%2%1%3"):trim())
	return str ~= "" and str
end

RantTooltip["SetUnit"] = function(self, unit)
	if (not unit) or (not UnitName(unit)) then
		GameTooltip:Hide()
		return
	end
	if GameTooltip.PreSetUnit then
		GameTooltip:PreSetUnit(unit)
	end
	self.factionInfo = self:GetUnitFaction(unit)
	self.unitInfo = self:GetUnitInfo(unit)
	GameTooltip:DeleteAllLines()
	for _, str in ipairs(unitTags) do
		local line = self:ReplaceTags(str, unit)
		if line then
			GameTooltip:AddLine(line)
		end
	end
	if GameTooltip.raidIcon then
		local raidIndex = GetRaidTargetIndex(unit)
		if raidIndex then
			GameTooltip.raidIcon:SetTexture("Interface\\TargetingFrame\\UI-RaidTargetingIcon_"..raidIndex)
		end
	end
	self:UpdateCombatIcon()
	for _, name in ipairs{"Health", "Power"} do
		local bar = GameTooltip[name.."Bar"]
		if bar then
			if _G["Unit"..name.."Max"](unit) ~= 0 then
				if (name == "Power" and bar.powerColors) or bar.unitColors then
					local color
					if bar.powerColors then
						local powerToken = select(2,UnitPowerType(unit))
						color = powerToken and RantTooltip.PowerColors[powerToken]
					else
						color = { GameTooltip_UnitColor(unit) }
					end
					if color then
						bar:SetStatusBarColor(unpack(color))
						if bar.bg and bar.bg.color then
							bar.bg:SetVertexColor(color[1]*bar.bg.color, color[2]*bar.bg.color, color[3]*bar.bg.color)
						end
					end
				end
				if name == "Power" then
					bar:Show()
					self:UpdatePowerBar()
				end
			end
		end
	end
	if GameTooltip.PostSetUnit then
		GameTooltip:PostSetUnit(unit)
	end
end

RantTooltip["GameTooltip:OnShow"] = function(self)
	if GameTooltip.PreOnShow then
		GameTooltip:PreOnShow()
	end
	local focus = GetMouseFocus()
	if focus then
		if focus == WorldFrame and (not UnitExists("mouseover")) then
			self.doodad = true
		end
		if focus:IsObjectType("CheckButton") then
			self:ClearAllPoints()
			self:SetPoint("BOTTOMLEFT", focus, "TOPRIGHT") 
		end
	end
	if GameTooltip.PostOnShow then
		GameTooltip:PostOnShow()
	end
end

RantTooltip["GameTooltip:OnHide"] = function(self)
	if GameTooltip.PreOnHide then
		GameTooltip:PreOnHide()
	end
	self.doodad = false
	if self.raidIcon then 
		self.raidIcon:SetTexture(nil)
	end
	if self.combatIcon then 
		self.combatIcon:Hide()
	end
	for var, tag in pairs(RantTooltip.TagEvents) do
		tag.last = nil
	end
	for _, bar in ipairs{self.HealthBar, self.PowerBar} do
		bar:Hide()
	end
	RantTooltip.Anchor[2] = RantTooltip.initialAnchor[2]
	RantTooltip.Anchor[3] = RantTooltip.initialAnchor[3]
	if GameTooltip.PostOnHide then
		GameTooltip:PostOnHide()
	end
end

RantTooltip["GameTooltip:OnTooltipSetUnit"] = function(self)
	RantTooltip:SetUnit(self.unit)
end

RantTooltip["GameTooltip:OnUpdate"] = function(self, elapsed)
	if RantTooltip.Anchor[1] == "CURSOR" and (RantTooltip.Anchor[2] + RantTooltip.Anchor[3]) ~= 0 then
		if self:GetAnchorType() == "ANCHOR_CURSOR" then
			local x, y = GetCursorPosition()
			local scale = UIParent:GetScale()
			local tooltipScale = self:GetScale()
			x = (x / scale) / tooltipScale
			y = (y / scale) / tooltipScale
			x = x - (self:GetWidth() / 2)
			self:ClearAllPoints()
			self:SetPoint("BOTTOMLEFT", UIParent, "BOTTOMLEFT", x + RantTooltip.Anchor[2], y + RantTooltip.Anchor[3])
		end
	end
	if RantTooltip.Anchor[1] ~= "CURSOR" and (not RantTooltip.fadeOut) then
		if GetMouseFocus() == WorldFrame and (not UnitExists("mouseover")) and (not self.doodad) then
			self:Hide()
		end
	end
	lastUpdate = (lastUpdate or 0) + elapsed
	if lastUpdate > 0.1 then
		lastUpdate = 0
		if self.unit then
			RantTooltip:UpdatePowerBar()
			RantTooltip:UpdateCombatIcon()
			for var, tag in pairs(RantTooltip.TagEvents) do
				if tag.update then
					RantTooltip:UpdateTag(var, tag)
				end
			end
		end
	end
end

RantTooltip["UpdateCombatIcon"] = function()
	if GameTooltip.combatIcon and GameTooltip.unit then
		if UnitAffectingCombat(GameTooltip.unit) and (not GameTooltip.combatIcon:IsShown()) then
			GameTooltip.combatIcon:Show()
		elseif (not UnitAffectingCombat(GameTooltip.unit)) and GameTooltip.combatIcon:IsShown() then
			GameTooltip.combatIcon:Hide()
		end
	end
end

RantTooltip["UpdateHealthBar"] = function()
	local bar, unit = GameTooltip.HealthBar, GameTooltip.unit
	if bar then
		local val = GameTooltipStatusBar:GetValue()
		local _, max = GameTooltipStatusBar:GetMinMaxValues()
		if GameTooltip.OverrideUpdateHealthBar then
			return GameTooltip:OverrideUpdateHealthBar(bar, val, max)
		end
		if GameTooltip.PreUpdateHealthBar then
			GameTooltip:PreUpdateHealthBar(bar, val, max)
		end
		bar:SetMinMaxValues(0, max)
		bar:SetValue(val)
		if bar.text then
			if val == 0 then
				bar.text:SetText("Dead")
				if GameTooltip.PowerBar then
					GameTooltip.PowerBar:Hide()
				end
			elseif unit and UnitIsGhost(unit) then
				bar.text:SetText("Ghost")
				if GameTooltip.PowerBar then
					GameTooltip.PowerBar:Hide()
				end
			else
				local text = RantTooltip:ReplaceTags(bar.Tags, unit, "Health")
				bar.text:SetText(text)
			end
		end
		if bar.unitColors and (not unit) then
			bar:SetStatusBarColor(1, 0, 0)
		end
		if not bar:IsShown() then
			bar:Show()
		end
		if GameTooltip.PostUpdateHealthBar then
			GameTooltip:PostUpdateHealthBar(bar, val, max)
		end
	end
end

RantTooltip["UpdatePowerBar"] = function()
	local bar = GameTooltip.PowerBar
	local cur = UnitPower(GameTooltip.unit)
	if bar and bar:IsShown() and bar:GetValue() ~= cur then
		local max = UnitPowerMax(GameTooltip.unit)
		if GameTooltip.OverrideUpdatePowerBar then
			return GameTooltip:OverrideUpdatePowerBar(bar, cur, max)
		end
		if GameTooltip.PreUpdatePowerBar then
			GameTooltip:PreUpdatePowerBar(bar, cur, max)
		end
		bar:SetMinMaxValues(0, max)
		bar:SetValue(cur)
		if bar.text then
			local text = bar.Tags
			text = RantTooltip:ReplaceTags(text, GameTooltip.unit, "Power")
			bar.text:SetText(text)
		end
	end
	if GameTooltip.PostUpdatePowerBar then
		GameTooltip:PostUpdatePowerBar(bar, cur, max)
	end
end

RantTooltip["GameTooltip:OnEvent"] = function(self, event)
	if event == "PLAYER_LOGIN" then
		RantTooltip:Initialize()
	else
		if self.unit and self:IsShown() then
			for var, tag in pairs(RantTooltip.TagEvents) do
				if tag.event and tag.event == event then
					RantTooltip:UpdateTag(var, tag)
				end
			end
		end
	end
end

_G["GameTooltip_UnitColor"] = function(unit)
	local r, g, b
	local reactionColors, classColors = RantTooltip.ReactionColors, RantTooltip.ClassColors
	if UnitIsPlayer(unit) then
		r, g, b = unpack(classColors[select(2,UnitClass(unit))])
	elseif UnitPlayerControlled(unit) then
		if UnitCanAttack(unit, "player") then
			-- Hostile players are red
			r, g, b = unpack(reactionColors[2])
		elseif UnitCanAttack("player", unit) then
			-- Players we can attack but which are not hostile are yellow
			r, g, b = unpack(reactionColors[4])
		elseif UnitIsPVP(unit) then
			-- Players we can assist but are PvP flagged are green
			r, g, b = unpack(reactionColors[6])
		else
			-- All other players are blue (the usual state on the "blue" server)
			r = 1.0
			g = 1.0
			b = 1.0
		end
	elseif UnitIsTapped(unit) and not UnitIsTappedByPlayer(unit) or not UnitIsConnected(unit) or UnitIsDead(unit) then
		r = 0.5
		g = 0.5
		b = 0.5
	else
		local reaction = UnitReaction(unit, "player")
		if reaction then
			r, g, b = unpack(reactionColors[reaction])
		else
			r = 1.0
			g = 1.0
			b = 1.0
		end
	end
	return r, g, b
end

_G.RantTooltip = RantTooltip

RantTooltip:OnLoad()