--RantTooltip Library
--Coded by Rum (Stuck)

local name = "RantTooltipLib"
local version = 0.02

local _G = getfenv(0)
local addon = _G["Rant"]

if addon and addon.libs[name] and (addon.libs[name].version or 0) >= version then
	return
elseif addon then
	addon.libs[name] = {}
	addon.libs[name].name = string.format("%s %s", addon.name, name)
	addon.libs[name].version = version
end

--Better version of GetUnit
local setUnit = GameTooltip.SetUnit
GameTooltip.SetUnit = function(self, arg1, ...)
	if arg1 and arg1 ~= "none" then
		self.unit = arg1
	end
	return setUnit(self, arg1, ...)
end

local onTooltipSetUnit = GameTooltip:GetScript("OnTooltipSetUnit")
GameTooltip:SetScript("OnTooltipSetUnit", function(self, ...)
	if GetMouseFocus() == WorldFrame then
		self.unit = "mouseover"
	end
	return onTooltipSetUnit(self, ...)
end)

GameTooltip:HookScript("OnHide", function(self)
	self.unit = nil
end)

GameTooltip["GetUnit"] = function(self)
	if self.unit then
		return UnitName(self.unit), self.unit
	end
end

--Clear the deleted variable from each line
GameTooltip:HookScript("OnTooltipCleared", function(self) 
	local lineCheck, i = true, 0
	while lineCheck do
		i = i + 1
		lineCheck = _G["GameTooltipTextLeft"..i]
		if lineCheck and lineCheck.deleted then
			lineCheck.deleted = nil
		end
	end
end)

--An adjustment for the tooltip statusbars (originally added for achievements) for whenever there is a line adjustment
GameTooltip["AdjustStatusBars"] = function(self, offset)
	local bar, i = GameTooltipStatusBar1, 1
	while bar do
		if bar:IsVisible() then
			local point = { bar:GetPoint() }
			local number = self:GetLine(point[2]) + offset
			point[2] = _G["GameTooltipTextLeft"..number]
			bar:SetPoint(unpack(point))
		end
		i = i + 1
		bar = _G["GameTooltipStatusBar"..i]
	end
end

local wrapText = function(text)
	if text:len() >= 45 then
		for str in text:gmatch("..%s..") do
			local last = text:gsub("(.*\n)(.*)", "%2")
			local startPos = last:find(str)
			if startPos and startPos >= 45 then
				local newText = last:sub(1, startPos+1).."\n"..last:sub(startPos+3, last:len())
				text = text:gsub(last, newText)
			end
		end
	end
	return text
end

--AddLine
local _AddLine = GameTooltip.AddLine
GameTooltip["AddLine"] = function(self, msg, r, g, b, wrap)
	local numLines = self:NumLines()
	for i = 1, numLines do
		local line = _G["GameTooltipTextLeft"..i]
		if line.deleted then
			msg = wrap and wrapText(msg) or msg
			line.deleted = nil
			line:SetText(msg)
			line:SetTextColor(r or 1, g or 0.82, b or 0)
			line:Show()
			return line
		end
	end
	_AddLine(self, msg, r, g, b, wrap)
	return self:GetLine(numLines+1)
end

--GetLine
GameTooltip["GetLine"] = function(self, num)
	if type(num) == "table" then
		num = num:GetName():gsub("GameTooltipTextLeft", "")
		num = tonumber(num)
		return num
	else
		return _G["GameTooltipTextLeft"..num]
	end
end

--InsertLine
GameTooltip["InsertLine"] = function(self, num, ...)
	local line, barNum = _G["GameTooltipTextLeft"..num]
	for i = self:NumLines(true), num, -1 do
		local cur = _G["GameTooltipTextLeft"..i]
		self:AddLine(cur:GetText(), cur:GetTextColor())
		cur.deleted = 1
	end
	line.deleted = 1
	self:AddLine(...)
	self:Show()
	barNum = select(2, self:FindLine(" ", true))
	if barNum and num <= barNum then
		self:AdjustStatusBars(1)
	end
end

--Delete All Lines
GameTooltip["DeleteAllLines"] = function(self)
	local numLines = self:NumLines(true)
	if numLines > 0 then
		for i = numLines, 1, -1 do
			self:DeleteLine(i)
		end
		if GameTooltipStatusBar1 and GameTooltipStatusBar1:IsVisible() then
			GameTooltip_ClearStatusBars(self)
		end
	end
end

--Delete Line
GameTooltip["DeleteLine"] = function(self, line, exact)
	local numLines, originalNum, barNum = self:NumLines()
	if type(line) == "number" then
		line = _G["GameTooltipTextLeft"..line]
	elseif type(line) == "string" then
		line = self:FindLine(line, exact)
	end
	if line then
		originalNum = self:GetLine(line)
		if line ~= _G["GameTooltipTextLeft"..numLines] then
			local number = self:GetLine(line)
			local tbl = {}
			for i = number+1, numLines do
				tbl[i-1] = _G["GameTooltipTextLeft"..i]
			end
			for k, v in pairs(tbl) do
				local text = v:GetText()
				local newLine = _G["GameTooltipTextLeft"..k]
				if v:GetStringHeight() > select(2,v:GetFont()) then
					text = wrapText(text)
				end
				newLine.deleted = v.deleted
				newLine:SetText(text)
				newLine:SetTextColor(v:GetTextColor())
			end
			line = _G["GameTooltipTextLeft"..numLines]
			if not line then return end
		end
		line.deleted = true
		line:SetText("")
		line:Hide()
		self:Show()
		barNum = select(2, self:FindLine(" ", true))
		if barNum and originalNum <= barNum then
			self:AdjustStatusBars(-1)
		end
	end
end

--Find a line
GameTooltip["FindLine"] = function(self, msg, exact, from) 
	if not msg then return end
	local lines = self:NumLines(true)
	for i = from or 1, lines do
		local line = _G["GameTooltipTextLeft"..i]
		local text = line and line:GetText()
		if text and ((exact and text == msg) or (not exact and text:find(msg))) then
			return line, i
		end
	end
end

--Num Lines
local _NumLines = GameTooltip.NumLines
GameTooltip["NumLines"] = function(self, ignoreDeleted)
	if ignoreDeleted then
		local line, realLines, i = true, 0, 0
		while line do
			i = i + 1
			line = _G[self:GetName().."TextLeft"..i]
			local text = line and line:GetText()
			if text and string.len(text) > 0 then
				realLines = realLines + 1
			else
				break
			end
		end
		return realLines
	end
	return _NumLines(self)
end
